@@ -50,7 +50,7 @@ defmodule CouchDBTest.Mixfile do
     [
       app: :couchdbtest,
       version: "0.1.0",
-      elixir: "~> 1.13",
+      elixir: "~> 1.15",
       lockfile: Path.expand("mix.lock", __DIR__),
       deps_path: Path.expand("src", __DIR__),
       build_path: Path.expand("_build", __DIR__),
@@ -93,7 +93,7 @@ defmodule CouchDBTest.Mixfile do
       {:jiffy, path: path("jiffy")},
       {:jwtf, path: path("jwtf")},
       {:ibrowse, path: path("ibrowse"), override: true},
-      {:credo, "~> 1.6.4", only: [:dev, :test, :integration], runtime: false}
+      {:credo, "~> 1.7.7", only: [:dev, :test, :integration], runtime: false}
     ]
 
     # Some deps may be missing during source check
@@ -146,17 +146,14 @@ defmodule CouchDBTest.Mixfile do
       "ssl_verify_fun",
       "unicode_util_compat",
       "b64url",
-      "bear",
+      "exxhash",
       "mochiweb",
       "snappy",
       "rebar",
       "proper",
       "mochiweb",
       "meck",
-      "khash",
-      "hyper",
-      "fauxton",
-      "folsom"
+      "fauxton"
     ]
 
     deps |> Enum.map(fn app -> "src/#{app}" end)
