@@ -39,6 +39,7 @@
     error_info/1,
     parse_form/1,
     json_body/1,
+    json_body/2,
     json_body_obj/1,
     body/1,
     doc_etag/1,
@@ -104,6 +105,8 @@ start_link(https) ->
     Port = config:get("ssl", "port", "6984"),
     {ok, Ciphers} = couch_util:parse_term(config:get("ssl", "ciphers", "undefined")),
     {ok, Versions} = couch_util:parse_term(config:get("ssl", "tls_versions", "undefined")),
+    {ok, SignatureAlgs} = couch_util:parse_term(config:get("ssl", "signature_algs", "undefined")),
+    {ok, NamedCurves} = couch_util:parse_term(config:get("ssl", "ecc_curves", "undefined")),
     {ok, SecureRenegotiate} = couch_util:parse_term(
         config:get("ssl", "secure_renegotiate", "undefined")
     ),
@@ -115,7 +118,9 @@ start_link(https) ->
             {password, config:get("ssl", "password", undefined)},
             {secure_renegotiate, SecureRenegotiate},
             {versions, Versions},
-            {ciphers, Ciphers}
+            {ciphers, Ciphers},
+            {signature_algs, SignatureAlgs},
+            {eccs, NamedCurves}
         ],
 
     case
@@ -235,7 +240,15 @@ handle_request_int(MochiReq) ->
     % for the path, use the raw path with the query string and fragment
     % removed, but URL quoting left intact
     RawUri = MochiReq:get(raw_path),
-    {"/" ++ Path, _, _} = mochiweb_util:urlsplit_path(RawUri),
+    {Path0, _, _} = mochiweb_util:urlsplit_path(RawUri),
+    Path =
+        case Path0 of
+            "/" ++ ValidPath ->
+                ValidPath;
+            _ ->
+                couch_stats:increment_counter([couchdb, httpd, aborted_requests]),
+                exit({shutdown, invalid_path})
+        end,
 
     % get requested path
     RequestedPath =
@@ -514,7 +527,7 @@ maybe_log(_HttpReq, #httpd_resp{should_log = false}) ->
 %% aren't sharded. So for now when a local db is specified as the source or
 %% the target, it's hacked to make it a full url and treated as a remote.
 possibly_hack(#httpd{path_parts = [<<"_replicate">>]} = Req) ->
-    {Props0} = chttpd:json_body_obj(Req),
+    {Props0} = json_body_obj(Req),
     Props1 = fix_uri(Req, Props0, <<"source">>),
     Props2 = fix_uri(Req, Props1, <<"target">>),
     Req#httpd{req_body = {Props2}};
@@ -794,14 +807,17 @@ body(#httpd{mochi_req = MochiReq, req_body = ReqBody}) ->
 validate_ctype(Req, Ctype) ->
     couch_httpd:validate_ctype(Req, Ctype).
 
-json_body(#httpd{req_body = undefined} = Httpd) ->
+json_body(#httpd{} = Httpd) ->
+    json_body(Httpd, []).
+
+json_body(#httpd{req_body = undefined} = Httpd, JsonDecodeOptions) ->
     case body(Httpd) of
         undefined ->
             throw({bad_request, "Missing request body"});
         Body ->
-            ?JSON_DECODE(maybe_decompress(Httpd, Body))
+            ?JSON_DECODE(maybe_decompress(Httpd, Body), JsonDecodeOptions)
     end;
-json_body(#httpd{req_body = ReqBody}) ->
+json_body(#httpd{req_body = ReqBody}, _JsonDecodeOptions) ->
     ReqBody.
 
 json_body_obj(Httpd) ->
@@ -814,14 +830,13 @@ doc_etag(#doc{id = Id, body = Body, revs = {Start, [DiskRev | _]}}) ->
     couch_httpd:doc_etag(Id, Body, {Start, DiskRev}).
 
 make_etag(Term) ->
-    <<SigInt:128/integer>> = couch_hash:md5_hash(term_to_binary(Term)),
-    list_to_binary(io_lib:format("\"~.36B\"", [SigInt])).
+    couch_httpd:make_etag(Term).
 
 etag_match(Req, CurrentEtag) when is_binary(CurrentEtag) ->
     etag_match(Req, binary_to_list(CurrentEtag));
 etag_match(Req, CurrentEtag) ->
     EtagsToMatch0 = string:tokens(
-        chttpd:header_value(Req, "If-None-Match", ""), ", "
+        header_value(Req, "If-None-Match", ""), ", "
     ),
     EtagsToMatch = lists:map(fun strip_weak_prefix/1, EtagsToMatch0),
     lists:member(CurrentEtag, EtagsToMatch).
@@ -836,7 +851,7 @@ etag_respond(Req, CurrentEtag, RespFun) ->
         true ->
             % the client has this in their cache.
             Headers = [{"ETag", CurrentEtag}],
-            chttpd:send_response(Req, 304, Headers, <<>>);
+            send_response(Req, 304, Headers, <<>>);
         false ->
             % Run the function.
             RespFun()
@@ -967,7 +982,7 @@ send_delayed_error(#delayed_resp{resp = Resp, req = Req}, Reason) ->
 close_delayed_json_object(Resp, Buffer, Terminator, 0) ->
     % Use a separate chunk to close the streamed array to maintain strict
     % compatibility with earlier versions. See COUCHDB-2724
-    {ok, R1} = chttpd:send_delayed_chunk(Resp, Buffer),
+    {ok, R1} = send_delayed_chunk(Resp, Buffer),
     send_delayed_chunk(R1, Terminator);
 close_delayed_json_object(Resp, Buffer, Terminator, _Threshold) ->
     send_delayed_chunk(Resp, [Buffer | Terminator]).
@@ -1017,7 +1032,7 @@ start_delayed_response(#delayed_resp{} = DelayedResp) ->
     {ok, DelayedResp}.
 
 buffer_response(Req) ->
-    case chttpd:qs_value(Req, "buffer_response") of
+    case qs_value(Req, "buffer_response") of
         "false" ->
             false;
         "true" ->
@@ -1084,6 +1099,8 @@ error_info({bad_ctype, Reason}) ->
     {415, <<"bad_content_type">>, Reason};
 error_info(requested_range_not_satisfiable) ->
     {416, <<"requested_range_not_satisfiable">>, <<"Requested range not satisfiable">>};
+error_info({expectation_failed, Reason}) ->
+    {417, <<"expectation_failed">>, Reason};
 error_info({error, {illegal_database_name, Name}}) ->
     Message =
         <<"Name: '", Name/binary, "'. Only lowercase characters (a-z), ",
@@ -1105,6 +1122,10 @@ error_info({error, <<"endpoint has an invalid url">> = Reason}) ->
     {400, <<"invalid_replication">>, Reason};
 error_info({error, <<"proxy has an invalid url">> = Reason}) ->
     {400, <<"invalid_replication">>, Reason};
+error_info({method_not_allowed, Reason}) ->
+    {405, <<"method_not_allowed">>, Reason};
+error_info({gone, Reason}) ->
+    {410, <<"gone">>, Reason};
 error_info({missing_stub, Reason}) ->
     {412, <<"missing_stub">>, Reason};
 error_info(request_entity_too_large) ->
@@ -1123,6 +1144,8 @@ error_info(all_workers_died) ->
         "Nodes are unable to service this "
         "request due to overloading or maintenance mode."
     >>};
+error_info({internal_server_error, Reason}) ->
+    {500, <<"internal_server_error">>, Reason};
 error_info(not_implemented) ->
     {501, <<"not_implemented">>, <<"this feature is not yet implemented">>};
 error_info(timeout) ->
@@ -1132,6 +1155,8 @@ error_info(timeout) ->
     >>};
 error_info({service_unavailable, Reason}) ->
     {503, <<"service unavailable">>, Reason};
+error_info({insufficient_storage, Reason}) ->
+    {507, <<"insufficent_storage">>, Reason};
 error_info({timeout, _Reason}) ->
     error_info(timeout);
 error_info({'EXIT', {Error, _Stack}}) ->
@@ -1337,7 +1362,7 @@ send_chunked_error(Resp, Error) ->
     send_chunk(Resp, []).
 
 send_redirect(Req, Path) ->
-    Headers = [{"Location", chttpd:absolute_uri(Req, Path)}],
+    Headers = [{"Location", absolute_uri(Req, Path)}],
     send_response(Req, 301, Headers, <<>>).
 
 server_header() ->
@@ -1422,7 +1447,7 @@ stack_trace_id(Stack) ->
     {"X-Couch-Stack-Hash", stack_hash(Stack)}.
 
 stack_hash(Stack) ->
-    erlang:crc32(term_to_binary(Stack)).
+    erlang:crc32(?term_to_bin(Stack)).
 
 %% @doc CouchDB uses a chunked transfer-encoding to stream responses to
 %% _all_docs, _changes, _view and other similar requests. This configuration
