@@ -18,7 +18,6 @@
 
 -module(config).
 -behaviour(gen_server).
--vsn(1).
 
 -export([start_link/1, stop/0, reload/0]).
 
@@ -31,33 +30,40 @@
 -export([get_float/3, set_float/3, set_float/4]).
 -export([get_boolean/3, set_boolean/3, set_boolean/4]).
 
--export([features/0, enable_feature/1, disable_feature/1]).
+-export([features/0, enable_feature/1, disable_feature/1, is_enabled/1]).
+
+-export([node_name/0, check_distribution_mode/0]).
 
 -export([listen_for_changes/2]).
 -export([subscribe_for_changes/1]).
--export([parse_ini_file/1]).
 
--export([init/1, terminate/2, code_change/3]).
+-export([init/1]).
 -export([handle_call/3, handle_cast/2, handle_info/2]).
 
 -export([is_sensitive/2]).
 
--define(FEATURES, "features").
+-define(FEATURES, features).
+
+-define(NODE_NAME, node_name).
 
 -define(TIMEOUT, 30000).
 -define(INVALID_SECTION, <<"Invalid configuration section">>).
 -define(INVALID_KEY, <<"Invalid configuration key">>).
 -define(INVALID_VALUE, <<"Invalid configuration value">>).
--define(DELETE, delete).
+
+-include("config.hrl").
 
 -record(config, {
-    notify_funs = [],
-    ini_files = undefined,
-    write_filename = undefined
+    % Mix of *.ini files and *.d dirs
+    ini_files_dirs,
+    % Just *.ini files: original *.ini + expanded *.d dirs
+    ini_files,
+    % The file we write config values to
+    write_filename
 }).
 
-start_link(IniFiles) ->
-    gen_server:start_link({local, ?MODULE}, ?MODULE, IniFiles, []).
+start_link(IniFilesDirs) ->
+    gen_server:start_link({local, ?MODULE}, ?MODULE, IniFilesDirs, []).
 
 stop() ->
     gen_server:cast(?MODULE, stop).
@@ -212,23 +218,68 @@ delete(Section, Key, Persist, Reason) when is_list(Section), is_list(Key) ->
     ).
 
 features() ->
-    application:get_env(config, enabled_features, []).
+    Map = persistent_term:get({?MODULE, ?FEATURES}, #{}),
+    lists:sort(maps:keys(Map)).
 
 enable_feature(Feature) when is_atom(Feature) ->
-    application:set_env(
-        config,
-        enabled_features,
-        lists:usort([Feature | features()]),
-        [{persistent, true}]
-    ).
+    Map = persistent_term:get({?MODULE, ?FEATURES}, #{}),
+    Map1 = Map#{Feature => true},
+    persistent_term:put({?MODULE, ?FEATURES}, Map1).
 
 disable_feature(Feature) when is_atom(Feature) ->
-    application:set_env(
-        config,
-        enabled_features,
-        features() -- [Feature],
-        [{persistent, true}]
-    ).
+    Map = persistent_term:get({?MODULE, ?FEATURES}, #{}),
+    Map1 = maps:remove(Feature, Map),
+    persistent_term:put({?MODULE, ?FEATURES}, Map1).
+
+is_enabled(Feature) when is_atom(Feature) ->
+    Map = persistent_term:get({?MODULE, ?FEATURES}, #{}),
+    maps:get(Feature, Map, false).
+
+% Some features like FIPS mode must be enabled earlier before couch, couch_epi
+% start up
+%
+enable_early_features() ->
+    % Mark FIPS if enabled
+    case crypto:info_fips() == enabled of
+        true ->
+            enable_feature(fips);
+        false ->
+            ok
+    end.
+
+% Use this function to return the configured node name. This name is guaranteed
+% to stay the same even during shutdown, for instance, when `node()` may start
+% returning `nonode@nohost` after the dist controller stops. Use in cases when
+% we intend to persist the name on disk, like the header epochs list or shard
+% maps.
+node_name() ->
+    % '%not_configured' is an invalid node name, unlike 'undefined'
+    case persistent_term:get({?MODULE, ?NODE_NAME}, '$not_configured') of
+        '$not_configured' ->
+            Name = node(),
+            ok = persistent_term:put({?MODULE, ?NODE_NAME}, Name),
+            Name;
+        Name when is_atom(Name) ->
+            Name
+    end.
+
+check_distribution_mode() ->
+    HasNameArg = init:get_argument(name) =/= error,
+    HasSNameArg = init:get_argument(sname) =/= error,
+    case {HasNameArg orelse HasSNameArg, node_name()} of
+        {true, 'nonode@nohost'} ->
+            {error, unexpected_standalone_mode};
+        {true, Name} when is_atom(Name), Name =:= node() ->
+            % expected in distributed mode
+            ok;
+        {true, Name} when is_atom(Name), Name =/= node() ->
+            {error, unexpected_node_name_change};
+        {false, 'nonode@nohost'} ->
+            % expected standalone mode
+            ok;
+        {false, Name} when is_atom(Name), Name =/= 'nonode@nohost' ->
+            {error, unexpected_distributed_mode}
+    end.
 
 listen_for_changes(CallbackModule, InitialState) ->
     config_listener_mon:subscribe(CallbackModule, InitialState).
@@ -236,25 +287,33 @@ listen_for_changes(CallbackModule, InitialState) ->
 subscribe_for_changes(Subscription) ->
     config_notifier:subscribe(Subscription).
 
-init(IniFiles) ->
+init(IniFilesDirs) ->
+    enable_early_features(),
     ets:new(?MODULE, [named_table, set, protected, {read_concurrency, true}]),
-    lists:map(
-        fun(IniFile) ->
-            {ok, ParsedIniValues} = parse_ini_file(IniFile),
-            ets:insert(?MODULE, ParsedIniValues)
+    IniFiles = expand_dirs(IniFilesDirs),
+    maps:foreach(
+        fun(K, V) ->
+            true = ets:insert(?MODULE, {K, V})
         end,
-        IniFiles
+        ini_map(IniFiles)
     ),
-    WriteFile =
-        case IniFiles of
-            [_ | _] -> lists:last(IniFiles);
-            _ -> undefined
-        end,
     debug_config(),
-    {ok, #config{ini_files = IniFiles, write_filename = WriteFile}}.
-
-terminate(_Reason, _State) ->
-    ok.
+    Config = #config{
+        ini_files_dirs = IniFilesDirs,
+        ini_files = IniFiles,
+        write_filename = get_write_file(IniFiles)
+    },
+    % This is one of the earliest calls from our
+    % applications. We both set and assert that we have the
+    % correct node name and distribution mode.
+    case check_distribution_mode() of
+        ok ->
+            {ok, Config};
+        {error, Msg} ->
+            erlang:display(Msg),
+            timer:sleep(500),
+            halt(1)
+    end.
 
 handle_call(all, _From, Config) ->
     Resp = lists:sort((ets:tab2list(?MODULE))),
@@ -262,34 +321,20 @@ handle_call(all, _From, Config) ->
 handle_call({set, Sec, Key, Val, Opts}, _From, Config) ->
     Persist = maps:get(persist, Opts, true),
     Reason = maps:get(reason, Opts, nil),
-    IsSensitive = is_sensitive(Sec, Key),
+    LogVal = maybe_conceal(Val, is_sensitive(Sec, Key)),
     case validate_config_update(Sec, Key, Val) of
-        {error, ValidationError} when IsSensitive ->
-            couch_log:error(
-                "~p: [~s] ~s = '****' rejected for reason ~p",
-                [?MODULE, Sec, Key, Reason]
-            ),
-            {reply, {error, ValidationError}, Config};
         {error, ValidationError} ->
             couch_log:error(
-                "~p: [~s] ~s = '~s' rejected for reason ~p",
-                [?MODULE, Sec, Key, Val, Reason]
+                "~p: [~s] ~s = ~s rejected for reason ~p",
+                [?MODULE, Sec, Key, LogVal, Reason]
             ),
             {reply, {error, ValidationError}, Config};
         ok ->
             true = ets:insert(?MODULE, {{Sec, Key}, Val}),
-            case IsSensitive of
-                false ->
-                    couch_log:notice(
-                        "~p: [~s] ~s set to ~s for reason ~p",
-                        [?MODULE, Sec, Key, Val, Reason]
-                    );
-                true ->
-                    couch_log:notice(
-                        "~p: [~s] ~s set to '****' for reason ~p",
-                        [?MODULE, Sec, Key, Reason]
-                    )
-            end,
+            couch_log:notice(
+                "~p: [~s] ~s set to ~s for reason ~p",
+                [?MODULE, Sec, Key, LogVal, Reason]
+            ),
             ConfigWriteReturn =
                 case {Persist, Config#config.write_filename} of
                     {true, undefined} ->
@@ -319,10 +364,17 @@ handle_call({delete, Sec, Key, Persist, Reason}, _From, Config) ->
             {true, undefined} ->
                 ok;
             {true, FileName} ->
-                config_writer:save_to_file({{Sec, Key}, ""}, FileName);
+                config_writer:save_to_file({{Sec, Key}, ?DELETE}, FileName);
             _ ->
                 ok
         end,
+    IniMap = ini_map(Config#config.ini_files),
+    case maps:find({Sec, Key}, IniMap) of
+        {ok, Val} ->
+            true = ets:insert(?MODULE, {{Sec, Key}, Val});
+        _ ->
+            ok
+    end,
     case ConfigDeleteReturn of
         ok ->
             Event = {config_change, Sec, Key, deleted, Persist},
@@ -331,68 +383,51 @@ handle_call({delete, Sec, Key, Persist, Reason}, _From, Config) ->
         Else ->
             {reply, Else, Config}
     end;
-handle_call(reload, _From, Config) ->
-    DiskKVs = lists:foldl(
-        fun(IniFile, DiskKVs0) ->
-            {ok, ParsedIniValues} = parse_ini_file(IniFile),
-            lists:foldl(
-                fun({K, V}, DiskKVs1) ->
-                    dict:store(K, V, DiskKVs1)
-                end,
-                DiskKVs0,
-                ParsedIniValues
-            )
-        end,
-        dict:new(),
-        Config#config.ini_files
-    ),
-    % Update ets with anything we just read
-    % from disk
-    dict:fold(
-        fun({Sec, Key} = K, V, _) ->
+handle_call(reload, _From, #config{} = Config) ->
+    #config{ini_files_dirs = IniFilesDirs} = Config,
+    IniFiles = expand_dirs(IniFilesDirs),
+    % Update ets with ini values.
+    IniMap = ini_map(IniFiles),
+    maps:foreach(
+        fun({Sec, Key} = K, V) ->
             VExisting = get(Sec, Key, V),
-            ets:insert(?MODULE, {K, V}),
+            true = ets:insert(?MODULE, {K, V}),
             case V =:= VExisting of
                 true ->
                     ok;
                 false ->
-                    case is_sensitive(Sec, Key) of
-                        false ->
-                            couch_log:notice(
-                                "Reload detected config change ~s.~s = ~p",
-                                [Sec, Key, V]
-                            );
-                        true ->
-                            couch_log:notice(
-                                "Reload detected config change ~s.~s = '****'",
-                                [Sec, Key]
-                            )
-                    end,
+                    couch_log:notice(
+                        "Reload detected config change ~s.~s = ~p",
+                        [Sec, Key, maybe_conceal(V, is_sensitive(Sec, Key))]
+                    ),
                     Event = {config_change, Sec, Key, V, true},
                     gen_event:sync_notify(config_event, Event)
             end
         end,
-        nil,
-        DiskKVs
+        IniMap
     ),
-    % And remove anything in ets that wasn't
-    % on disk.
+    % And remove anything in ets that wasn't on disk.
     ets:foldl(
-        fun({{Sec, Key} = K, _}, _) ->
-            case dict:is_key(K, DiskKVs) of
-                true ->
-                    ok;
-                false ->
-                    couch_log:notice("Reload deleting in-memory config ~s.~s", [Sec, Key]),
-                    ets:delete(?MODULE, K),
-                    Event = {config_change, Sec, Key, deleted, true},
-                    gen_event:sync_notify(config_event, Event)
-            end
+        fun
+            ({{Sec, Key} = K, _}, _) when not is_map_key(K, IniMap) ->
+                couch_log:notice(
+                    "Reload deleting in-memory config ~s.~s",
+                    [Sec, Key]
+                ),
+                ets:delete(?MODULE, K),
+                Event = {config_change, Sec, Key, deleted, true},
+                gen_event:sync_notify(config_event, Event);
+            (_, _) ->
+                ok
         end,
         nil,
         ?MODULE
     ),
-    {reply, ok, Config}.
+    Config1 = Config#config{
+        ini_files = IniFiles,
+        write_filename = get_write_file(IniFiles)
+    },
+    {reply, ok, Config1}.
 
 handle_cast(stop, State) ->
     {stop, normal, State};
@@ -403,8 +438,10 @@ handle_info(Info, State) ->
     couch_log:error("config:handle_info Info: ~p~n", [Info]),
     {noreply, State}.
 
-code_change(_OldVsn, State, _Extra) ->
-    {ok, State}.
+maybe_conceal(Value, _IsSensitive = false) ->
+    Value;
+maybe_conceal(_, _IsSensitive = true) ->
+    "'****'".
 
 is_sensitive(Section, Key) ->
     Sensitive = application:get_env(config, sensitive, #{}),
@@ -414,10 +451,21 @@ is_sensitive(Section, Key) ->
         _ -> false
     end.
 
+ini_map(IniFiles) ->
+    lists:foldl(
+        fun(IniFile, IniMap0) ->
+            {ok, Props} = parse_ini_file(IniFile),
+            IniMap = maps:from_list(Props),
+            maps:merge(IniMap0, IniMap)
+        end,
+        #{},
+        IniFiles
+    ).
+
 parse_ini_file(IniFile) ->
     IniBin = read_ini_file(IniFile),
     ParsedIniValues = parse_ini(IniBin),
-    {ok, lists:filter(fun delete_keys/1, ParsedIniValues)}.
+    {ok, ParsedIniValues}.
 
 parse_ini(IniBin) when is_binary(IniBin) ->
     Lines0 = re:split(IniBin, "\r\n|\n|\r|\032", [{return, list}]),
@@ -446,10 +494,9 @@ parse_fold(Line, {Section, KVs}) ->
         [_] ->
             % Failed to split on " = ", so try to split on "=".
             % If the line starts with "=" or it's not a KV pair, ignore it.
-            % An empty value emit the `delete` atom as a marker.
             case string:split(Line, "=") of
                 ["", _] -> {Section, KVs};
-                [K, ""] -> {Section, [{{Section, trim(K)}, ?DELETE} | KVs]};
+                [_, ""] -> {Section, KVs};
                 [K, V] -> {Section, [{{Section, trim(K)}, trim(V)} | KVs]};
                 [_] -> {Section, KVs}
             end
@@ -467,6 +514,25 @@ read_ini_file(IniFile) ->
             throw({startup_error, Msg})
     end.
 
+get_write_file([_ | _] = IniFiles) ->
+    lists:last(IniFiles);
+get_write_file(_) ->
+    undefined.
+
+% Takes a list of files and directories with *.ini files, and expands
+% the directories inline with individual *.ini files
+expand_dirs(IniFilesDirs) ->
+    lists:flatmap(fun expand_dirs_fmap/1, IniFilesDirs).
+
+expand_dirs_fmap(File) ->
+    case filelib:is_dir(File) of
+        true ->
+            Fun = fun(IniFile) -> filename:join(File, IniFile) end,
+            lists:map(Fun, lists:sort(filelib:wildcard("*.ini", File)));
+        false ->
+            [File]
+    end.
+
 remove_comments(Line) ->
     case trim(Line) of
         [$; | _] ->
@@ -479,14 +545,6 @@ remove_comments(Line) ->
             NoComments
     end.
 
-% Specially handle the ?DELETE marker
-%
-delete_keys({{Section, Key}, ?DELETE}) ->
-    ets:delete(?MODULE, {Section, Key}),
-    false;
-delete_keys({{_, _}, _}) ->
-    true.
-
 trim(String) ->
     % May look silly but we're using this quite a bit
     string:trim(String).
@@ -538,7 +596,7 @@ validate_config_update(Sec, Key, Val) ->
     end.
 
 -ifdef(TEST).
--include_lib("eunit/include/eunit.hrl").
+-include_lib("couch/include/couch_eunit.hrl").
 
 to_integer_test() ->
     ?assertEqual(1, to_integer(1)),
@@ -554,7 +612,8 @@ to_float_test() ->
     ?assertEqual(1.0, to_float(<<"1.0">>)),
     ?assertEqual(1.0, to_float("1.0")),
     ?assertEqual(-1.1, to_float("-01.1")),
-    ?assertEqual(0.0, to_float("-0.0")),
+    ?assertEqual(0.0, to_float("0.0")),
+    ?assertEquiv(0.0, to_float("-0.0")),
     ?assertEqual(0.0, to_float("+0.0")),
     ok.
 
@@ -680,17 +739,17 @@ parse_extra_equal_sign_test() ->
     ?assertEqual([{{"s", "==k=="}, "v"}], ini("[s]\n==k== = v")).
 
 parse_delete_test() ->
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk=")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk =")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk = ")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk= ")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk = ")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk = ;")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk  =\t;")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk=\n")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk = ; ;")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk = ;v")),
-    ?assertEqual([{{"s", "k"}, ?DELETE}], ini("[s]\nk = ;")).
+    ?assertEqual([], ini("[s]\nk=")),
+    ?assertEqual([], ini("[s]\nk =")),
+    ?assertEqual([], ini("[s]\nk = ")),
+    ?assertEqual([], ini("[s]\nk= ")),
+    ?assertEqual([], ini("[s]\nk = ")),
+    ?assertEqual([], ini("[s]\nk = ;")),
+    ?assertEqual([], ini("[s]\nk  =\t;")),
+    ?assertEqual([], ini("[s]\nk=\n")),
+    ?assertEqual([], ini("[s]\nk = ; ;")),
+    ?assertEqual([], ini("[s]\nk = ;v")),
+    ?assertEqual([], ini("[s]\nk = ;")).
 
 parse_comments_test() ->
     ?assertEqual([], ini("[s]\n;k=v")),
@@ -738,4 +797,16 @@ parse_multiple_kvs_test() ->
         ini("[s]\nk1=v1\ngarbage\n= more garbage\nk2=v2")
     ).
 
+read_non_existent_ini_file_test_() ->
+    {
+        setup,
+        fun() -> meck:expect(couch_log, error, fun(_, _) -> ok end) end,
+        fun(_) -> meck:unload() end,
+        [
+            ?_assertException(
+                throw, {startup_error, _}, read_ini_file("non-existent")
+            )
+        ]
+    }.
+
 -endif.
