@@ -19,7 +19,6 @@
 ]).
 
 -include_lib("couch/include/couch_eunit.hrl").
--include_lib("couch/include/couch_db.hrl").
 
 -define(TIMEOUT, 4000).
 -define(RESTART_TIMEOUT_IN_MILLISEC, 3000).
@@ -56,8 +55,7 @@
     FileName
 end).
 
--define(T(F), {erlang:fun_to_list(F), F}).
--define(FEXT(F), fun(_, _) -> F() end).
+-define(TEMP_CONFIG_DIR, "temp_config.d").
 
 setup() ->
     setup(?CONFIG_CHAIN).
@@ -115,6 +113,58 @@ handle_config_terminate(Self, Reason, {Pid, State}) ->
     Pid ! {config_msg, {Self, Reason, State}},
     ok.
 
+setup_ini() ->
+    Chain = [write_ini(F) || F <- ["default.ini", "local.ini", ?TEMP_CONFIG_DIR]],
+    {Chain, setup(Chain)}.
+
+teardown_ini({Chain, Apps}) ->
+    lists:foreach(fun delete_ini/1, Chain),
+    teardown(Apps).
+
+write_ini(FileOrDir) ->
+    case filename:extension(FileOrDir) of
+        ".ini" ->
+            Path = filename:join([?TEMPDIR, FileOrDir]),
+            Data = io_lib:format("[section]\nfile = ~s\n", [FileOrDir]),
+            ok = file:write_file(Path, Data),
+            Path;
+        ".d" ->
+            Path = filename:join([?TEMPDIR, FileOrDir]),
+            ok = filelib:ensure_path(Path),
+            Path
+    end.
+
+delete_ini(FileOrDir) ->
+    case filename:extension(FileOrDir) of
+        ".ini" ->
+            ok = file:delete(FileOrDir);
+        ".d" ->
+            ok = file:del_dir_r(FileOrDir)
+    end.
+
+config_delete_reload_restart_test_() ->
+    {
+        "Test consistency after set, delete, reload, and restart",
+        foreach,
+        fun setup_ini/0,
+        fun teardown_ini/1,
+        [
+            fun non_persistent_set_delete_reload_restart/0,
+            fun persistent_set_delete_reload_restart/0
+        ]
+    }.
+
+config_reload_find_new_init_files_test_() ->
+    {
+        "Reload should find newly added .ini file in .d dirs",
+        foreach,
+        fun setup_ini/0,
+        fun teardown_ini/1,
+        [
+            fun reload_finds_new_ini_file/0
+        ]
+    }.
+
 config_get_test_() ->
     {
         "Config get tests",
@@ -144,7 +194,8 @@ config_set_test_() ->
             [
                 fun should_update_option/0,
                 fun should_create_new_section/0,
-                fun should_fail_to_set_binary_value/0
+                fun should_fail_to_set_binary_value/0,
+                fun should_fail_to_set_invalid_sensitive_key/0
             ]
         }
     }.
@@ -390,6 +441,16 @@ should_fail_to_set_binary_value() ->
         config:set(<<"a">>, <<"b">>, <<"c">>, false)
     ).
 
+should_fail_to_set_invalid_sensitive_key() ->
+    ?assertEqual(
+        {error, <<"Invalid configuration key">>},
+        config:set("admins", "[]", "val")
+    ),
+    ?assertEqual(
+        "'****'",
+        lists:nth(4, meck:capture(first, couch_log, error, ['_', '_'], 2))
+    ).
+
 should_return_undefined_atom_after_option_deletion() ->
     ?assertEqual(ok, config:delete("mock_log", "level", false)),
     ?assertEqual(undefined, config:get("mock_log", "level")).
@@ -441,8 +502,8 @@ should_write_changes(_, _) ->
         ?assertEqual("5986", config:get("httpd", "port")),
         ?assertEqual(ok, config:set("httpd", "port", "8080")),
         ?assertEqual("8080", config:get("httpd", "port")),
-        ?assertEqual(ok, config:delete("httpd", "bind_address", "8080")),
-        ?assertEqual(undefined, config:get("httpd", "bind_address"))
+        ?assertEqual(ok, config:delete("httpd", "bind_address")),
+        ?assertEqual("127.0.0.1", config:get("httpd", "bind_address"))
     end).
 
 should_ensure_default_wasnt_modified(_, _) ->
@@ -461,16 +522,12 @@ should_ensure_that_no_ini_files_loaded() ->
     ?assertEqual(0, length(config:all())).
 
 should_create_non_persistent_option() ->
-    ?_test(begin
-        ?assertEqual(ok, config:set("httpd", "port", "80", false)),
-        ?assertEqual("80", config:get("httpd", "port"))
-    end).
+    ?assertEqual(ok, config:set("httpd", "port", "80", false)),
+    ?assertEqual("80", config:get("httpd", "port")).
 
 should_create_persistent_option() ->
-    ?_test(begin
-        ?assertEqual(ok, config:set("httpd", "bind_address", "127.0.0.1")),
-        ?assertEqual("127.0.0.1", config:get("httpd", "bind_address"))
-    end).
+    ?assertEqual(ok, config:set("httpd", "bind_address", "127.0.0.1")),
+    ?assertEqual("127.0.0.1", config:get("httpd", "bind_address")).
 
 should_handle_value_change({_Apps, Pid}) ->
     ?_test(begin
@@ -651,11 +708,14 @@ should_enable_features() ->
 
     ?assertEqual(ok, config:enable_feature(snek)),
     ?assertEqual([snek], config:features()),
+    ?assert(config:is_enabled(snek)),
 
     ?assertEqual(ok, config:enable_feature(snek)),
     ?assertEqual([snek], config:features()),
 
     ?assertEqual(ok, config:enable_feature(dogo)),
+    ?assert(config:is_enabled(dogo)),
+    ?assert(config:is_enabled(snek)),
     ?assertEqual([dogo, snek], config:features()).
 
 should_disable_features() ->
@@ -666,9 +726,11 @@ should_disable_features() ->
     ?assertEqual([snek], config:features()),
 
     ?assertEqual(ok, config:disable_feature(snek)),
+    ?assertNot(config:is_enabled(snek)),
     ?assertEqual([], config:features()),
 
     ?assertEqual(ok, config:disable_feature(snek)),
+    ?assertNot(config:is_enabled(snek)),
     ?assertEqual([], config:features()).
 
 should_keep_features_on_config_restart() ->
@@ -678,6 +740,7 @@ should_keep_features_on_config_restart() ->
     config:enable_feature(snek),
     ?assertEqual([snek], config:features()),
     with_process_restart(config),
+    ?assert(config:is_enabled(snek)),
     ?assertEqual([snek], config:features()).
 
 should_notify_on_config_reload(Subscription, {_Apps, Pid}) ->
@@ -708,6 +771,88 @@ should_notify_on_config_reload_flush(Subscription, {_Apps, Pid}) ->
         end)
     }.
 
+persistent_set_delete_reload_restart() ->
+    Persist = true,
+    ?assertEqual("local.ini", config:get("section", "file")),
+    ?assertEqual(ok, config:set("section", "file", "memory", Persist)),
+    ?assertEqual("memory", config:get("section", "file")),
+    ?assertEqual(ok, config:delete("section", "file", Persist)),
+    ?assertEqual("default.ini", config:get("section", "file")),
+    ?assertEqual(ok, config:reload()),
+    ?assertEqual("default.ini", config:get("section", "file")),
+    with_process_restart(config),
+    % Avoid race with config loading .ini files
+    wait_config_get("section", "file", "default.ini"),
+    ?assertEqual("default.ini", config:get("section", "file")).
+
+non_persistent_set_delete_reload_restart() ->
+    Persist = false,
+    ?assertEqual("local.ini", config:get("section", "file")),
+    ?assertEqual(ok, config:set("section", "file", "memory", Persist)),
+    ?assertEqual("memory", config:get("section", "file")),
+    ?assertEqual(ok, config:delete("section", "file", Persist)),
+    ?assertEqual("local.ini", config:get("section", "file")),
+    ?assertEqual(ok, config:reload()),
+    ?assertEqual("local.ini", config:get("section", "file")),
+    with_process_restart(config),
+    % Avoid race with config loading .ini files
+    wait_config_get("section", "file", "local.ini"),
+    ?assertEqual("local.ini", config:get("section", "file")).
+
+reload_finds_new_ini_file() ->
+    config:set("foo", "bar", "baz"),
+    ?assertEqual(ok, config:reload()),
+    ?assertEqual("baz", config:get("foo", "bar")),
+    DirPath = filename:join([?TEMPDIR, ?TEMP_CONFIG_DIR]),
+    ?assert(filelib:is_dir(DirPath)),
+    IniPath = filename:join(DirPath, "10-extra.ini"),
+    ok = file:write_file(IniPath, "[foo]\nbar = zab\n"),
+    ?assertEqual(ok, config:reload()),
+    wait_config_get("foo", "bar", "zab"),
+    config:set("foo", "bar", "zag"),
+    % 10-extra.ini is also the file we write to
+    ?assertEqual({ok, <<"[foo]\nbar = zag\n">>}, file:read_file(IniPath)).
+
+node_name_test_() ->
+    {
+        foreach,
+        fun setup_node_name/0,
+        fun teardown_node_name/1,
+        [
+            ?TDEF_FE(t_node_name),
+            ?TDEF_FE(t_check_distributed_mode)
+        ]
+    }.
+
+setup_node_name() ->
+    persistent_term:erase({config, node_name}).
+
+teardown_node_name(_) ->
+    persistent_term:erase({config, node_name}).
+
+t_node_name(_) ->
+    ?assertEqual(not_there, persistent_term:get({config, node_name}, not_there)),
+    ?assertEqual(node(), config:node_name()),
+    ?assertEqual(node(), persistent_term:get({config, node_name})),
+    ?assertEqual(node(), config:node_name()).
+
+t_check_distributed_mode(_) ->
+    % We can't mock init so we do the best we can with standalone args
+    persistent_term:put({config, node_name}, 'nonode@nohost'),
+    ?assertEqual(ok, config:check_distribution_mode()),
+    persistent_term:put({config, node_name}, 'foo@127.0.0.1'),
+    ?assertEqual({error, unexpected_distributed_mode}, config:check_distribution_mode()).
+
+wait_config_get(Sec, Key, Val) ->
+    test_util:wait(
+        fun() ->
+            case config:get(Sec, Key) of
+                V when V =:= Val -> ok;
+                _ -> wait
+            end
+        end
+    ).
+
 spawn_config_listener() ->
     Self = self(),
     Pid = erlang:spawn(fun() ->
