@@ -1320,7 +1320,7 @@ update_docs(Db, Docs0, Options, ?REPLICATED_CHANGES) ->
         )
     end,
 
-    {ok, DocBuckets, NonRepDocs, DocErrors} =
+    {ok, DocBuckets, LocalDocs, DocErrors} =
         before_docs_update(Db, Docs, PrepValidateFun, ?REPLICATED_CHANGES),
 
     DocBuckets2 = [
@@ -1333,11 +1333,20 @@ update_docs(Db, Docs0, Options, ?REPLICATED_CHANGES) ->
     {ok, _} = write_and_commit(
         Db,
         DocBuckets2,
-        NonRepDocs,
-        [merge_conflicts | Options]
+        LocalDocs,
+        [?REPLICATED_CHANGES | Options]
     ),
     {ok, DocErrors};
 update_docs(Db, Docs0, Options, ?INTERACTIVE_EDIT) ->
+    BlockInteractiveDatabaseWrites = couch_disk_monitor:block_interactive_database_writes(),
+    if
+        BlockInteractiveDatabaseWrites ->
+            {ok, [{insufficient_storage, <<"database_dir is too full">>} || _ <- Docs0]};
+        true ->
+            update_docs_interactive(Db, Docs0, Options)
+    end.
+
+update_docs_interactive(Db, Docs0, Options) ->
     Docs = tag_docs(Docs0),
 
     AllOrNothing = lists:member(all_or_nothing, Options),
@@ -1352,7 +1361,7 @@ update_docs(Db, Docs0, Options, ?INTERACTIVE_EDIT) ->
         )
     end,
 
-    {ok, DocBuckets, NonRepDocs, DocErrors} =
+    {ok, DocBuckets, LocalDocs, DocErrors} =
         before_docs_update(Db, Docs, PrepValidateFun, ?INTERACTIVE_EDIT),
 
     if
@@ -1372,7 +1381,7 @@ update_docs(Db, Docs0, Options, ?INTERACTIVE_EDIT) ->
         true ->
             Options2 =
                 if
-                    AllOrNothing -> [merge_conflicts];
+                    AllOrNothing -> [?REPLICATED_CHANGES];
                     true -> []
                 end ++ Options,
             DocBuckets2 = [
@@ -1392,7 +1401,7 @@ update_docs(Db, Docs0, Options, ?INTERACTIVE_EDIT) ->
             {ok, CommitResults} = write_and_commit(
                 Db,
                 DocBuckets3,
-                NonRepDocs,
+                LocalDocs,
                 Options2
             ),
 
@@ -1451,14 +1460,14 @@ collect_results(Pid, MRef, ResultsAcc) ->
 write_and_commit(
     #db{main_pid = Pid, user_ctx = Ctx} = Db,
     DocBuckets1,
-    NonRepDocs,
+    LocalDocs,
     Options
 ) ->
     DocBuckets = prepare_doc_summaries(Db, DocBuckets1),
-    MergeConflicts = lists:member(merge_conflicts, Options),
+    ReplicatedChanges = lists:member(?REPLICATED_CHANGES, Options),
     MRef = erlang:monitor(process, Pid),
     try
-        Pid ! {update_docs, self(), DocBuckets, NonRepDocs, MergeConflicts},
+        Pid ! {update_docs, self(), DocBuckets, LocalDocs, ReplicatedChanges},
         case collect_results_with_metrics(Pid, MRef, []) of
             {ok, Results} ->
                 {ok, Results};
@@ -1473,7 +1482,7 @@ write_and_commit(
                 % We only retry once
                 DocBuckets3 = prepare_doc_summaries(Db2, DocBuckets2),
                 close(Db2),
-                Pid ! {update_docs, self(), DocBuckets3, NonRepDocs, MergeConflicts},
+                Pid ! {update_docs, self(), DocBuckets3, LocalDocs, ReplicatedChanges},
                 case collect_results_with_metrics(Pid, MRef, []) of
                     {ok, Results} -> {ok, Results};
                     retry -> throw({update_error, compaction_retry})
@@ -1521,7 +1530,7 @@ before_docs_update(#db{validate_doc_funs = VDFuns} = Db, Docs, PVFun, UpdateType
         (#doc{id = <<?LOCAL_DOC_PREFIX, _/binary>>}) -> true;
         (_) -> false
     end,
-    {NonRepDocs, Docs2} = lists:partition(IsLocal, Docs),
+    {LocalDocs, Docs2} = lists:partition(IsLocal, Docs),
 
     BucketList = group_alike_docs(Docs2),
 
@@ -1551,9 +1560,9 @@ before_docs_update(#db{validate_doc_funs = VDFuns} = Db, Docs, PVFun, UpdateType
             {DocBuckets2, DocErrors} = PVFun(Db, DocBuckets, ExistingDocs),
             % remove empty buckets
             DocBuckets3 = [Bucket || Bucket <- DocBuckets2, Bucket /= []],
-            {ok, DocBuckets3, NonRepDocs, DocErrors};
+            {ok, DocBuckets3, LocalDocs, DocErrors};
         false ->
-            {ok, DocBuckets, NonRepDocs, []}
+            {ok, DocBuckets, LocalDocs, []}
     end.
 
 set_new_att_revpos(#doc{revs = {RevPos, _Revs}, atts = Atts0} = Doc) ->
@@ -1599,16 +1608,6 @@ compressible_att_type(MimeType) ->
         [T || T <- TypeExpList, T /= []]
     ).
 
-% From RFC 2616 3.6.1 - Chunked Transfer Coding
-%
-%   In other words, the origin server is willing to accept
-%   the possibility that the trailer fields might be silently
-%   discarded along the path to the client.
-%
-% I take this to mean that if "Trailers: Content-MD5\r\n"
-% is present in the request, but there is no Content-MD5
-% trailer, we're free to ignore this inconsistency and
-% pretend that no Content-MD5 exists.
 with_stream(Db, Att, Fun) ->
     [InMd5, Type, Enc] = couch_att:fetch([md5, type, encoding], Att),
     BufferSize = config:get_integer(
@@ -1631,19 +1630,10 @@ with_stream(Db, Att, Fun) ->
                 [{buffer_size, BufferSize}]
         end,
     {ok, OutputStream} = open_write_stream(Db, Options),
-    ReqMd5 =
-        case Fun(OutputStream) of
-            {md5, FooterMd5} ->
-                case InMd5 of
-                    md5_in_footer -> FooterMd5;
-                    _ -> InMd5
-                end;
-            _ ->
-                InMd5
-        end,
+    Fun(OutputStream),
     {StreamEngine, Len, IdentityLen, Md5, IdentityMd5} =
         couch_stream:close(OutputStream),
-    couch_util:check_md5(IdentityMd5, ReqMd5),
+    couch_util:check_md5(IdentityMd5, InMd5),
     {AttLen, DiskLen, NewEnc} =
         case Enc of
             identity ->
