@@ -13,7 +13,6 @@
 -module(couch_proc_manager).
 -behaviour(gen_server).
 -behaviour(config_listener).
--vsn(3).
 
 -export([
     start_link/0,
@@ -34,8 +33,7 @@
     terminate/2,
     handle_call/3,
     handle_cast/2,
-    handle_info/2,
-    code_change/3
+    handle_info/2
 ]).
 
 -export([
@@ -81,13 +79,17 @@ get_proc(#doc{body = {Props}} = DDoc, DbKey, {_DDocId, _Rev} = DDocKey) ->
     Lang = couch_util:to_binary(LangStr),
     Client = #client{lang = Lang, ddoc = DDoc, db_key = DbKey, ddoc_key = DDocKey},
     Timeout = get_os_process_timeout(),
-    gen_server:call(?MODULE, {get_proc, Client}, Timeout).
+    Res = gen_server:call(?MODULE, {get_proc, Client}, Timeout),
+    couch_stats:increment_counter([couchdb, query_server, acquired_processes]),
+    Res.
 
 get_proc(LangStr) ->
     Lang = couch_util:to_binary(LangStr),
     Client = #client{lang = Lang},
     Timeout = get_os_process_timeout(),
-    gen_server:call(?MODULE, {get_proc, Client}, Timeout).
+    Res = gen_server:call(?MODULE, {get_proc, Client}, Timeout),
+    couch_stats:increment_counter([couchdb, query_server, acquired_processes]),
+    Res.
 
 ret_proc(#proc{} = Proc) ->
     gen_server:call(?MODULE, {ret_proc, Proc}, infinity).
@@ -141,6 +143,7 @@ init([]) ->
     ets:insert(?SERVERS, get_servers_from_env("COUCHDB_NATIVE_QUERY_SERVER_")),
     ets:insert(?SERVERS, [{"QUERY", {mango_native_proc, start_link, []}}]),
     maybe_configure_erlang_native_servers(),
+    configure_js_engine(couch_server:get_js_engine()),
 
     {ok, #state{
         config = get_proc_config(),
@@ -233,7 +236,7 @@ handle_info(shutdown, State) ->
     {stop, shutdown, State};
 handle_info({'EXIT', Pid, {spawn_ok, Proc0, undefined = _From}}, State) ->
     % Use ets:take/2 to assert that opener existed before removing. Also assert that
-    % the pid matches and the the client was a bogus client
+    % the pid matches and the client was a bogus client
     [{Pid, #client{from = undefined}}] = ets:take(?OPENING, Pid),
     Proc = Proc0#proc{client = undefined},
     link(Proc#proc.pid),
@@ -275,9 +278,6 @@ handle_info(restart_config_listener, State) ->
 handle_info(_Msg, State) ->
     {noreply, State}.
 
-code_change(_OldVsn, #state{} = State, _Extra) ->
-    {ok, State}.
-
 handle_config_terminate(_, stop, _) ->
     ok;
 handle_config_terminate(_Server, _Reason, _State) ->
@@ -536,6 +536,25 @@ maybe_configure_erlang_native_servers() ->
             ok
     end.
 
+configure_js_engine(<<"quickjs">>) ->
+    ets:insert(?SERVERS, [
+        {"JAVASCRIPT", couch_quickjs:mainjs_cmd()},
+        {"COFFEESCRIPT", couch_quickjs:coffee_cmd()},
+        {"JAVASCRIPT_QUICKJS", couch_quickjs:mainjs_cmd()}
+    ]),
+    case couch_server:with_spidermonkey() of
+        true ->
+            SM_ENV = os:getenv("COUCHDB_QUERY_SERVER_JAVASCRIPT"),
+            ets:insert(?SERVERS, {"JAVASCRIPT_SPIDERMONKEY", SM_ENV});
+        false ->
+            ok
+    end;
+configure_js_engine(<<"spidermonkey">>) ->
+    ets:insert(?SERVERS, [
+        {"JAVASCRIPT_QUICKJS", couch_quickjs:mainjs_cmd()},
+        {"JAVASCRIPT_SPIDERMONKEY", os:getenv("COUCHDB_QUERY_SERVER_JAVASCRIPT")}
+    ]).
+
 new_proc_int(From, Lang) when is_binary(Lang) ->
     LangStr = binary_to_list(Lang),
     case get_query_server(LangStr) of
