@@ -13,12 +13,11 @@
 -module(couch_server).
 -behaviour(gen_server).
 -behaviour(config_listener).
--vsn(3).
 
 -export([open/2, create/2, delete/2, get_version/0, get_version/1, get_git_sha/0, get_uuid/0]).
 -export([all_databases/0, all_databases/2]).
 -export([init/1, handle_call/3, sup_start_link/1]).
--export([handle_cast/2, code_change/3, handle_info/2, terminate/2]).
+-export([handle_cast/2, handle_info/2, terminate/2]).
 -export([dev_start/0, is_admin/2, has_admins/0, get_stats/0]).
 -export([close_db_if_idle/1]).
 -export([delete_compaction_files/1]).
@@ -29,6 +28,7 @@
 -export([lock/2, unlock/1]).
 -export([db_updated/1]).
 -export([num_servers/0, couch_server/1, couch_dbs_pid_to_name/1, couch_dbs/1]).
+-export([get_js_engine/0, with_spidermonkey/0]).
 -export([aggregate_queue_len/0, get_spidermonkey_version/0]).
 -export([names/0]).
 -export([try_lock/2, unlock/2]).
@@ -91,8 +91,19 @@ get_stats() ->
         lists:foldl(Fun, {0, 0}, lists:seq(1, num_servers())),
     [{start_time, ?l2b(Time)}, {dbs_open, Open}].
 
+get_js_engine() ->
+    list_to_binary(config:get("couchdb", "js_engine", ?COUCHDB_JS_ENGINE)).
+
 get_spidermonkey_version() -> list_to_binary(?COUCHDB_SPIDERMONKEY_VERSION).
 
+with_spidermonkey() ->
+    % The case match is just an extra assert that we got a correctly configured
+    % value from rebar config script
+    case ?COUCHDB_WITH_SPIDERMONKEY of
+        true -> true;
+        false -> false
+    end.
+
 sup_start_link(N) ->
     gen_server:start_link({local, couch_server(N)}, couch_server, [N], []).
 
@@ -274,14 +285,6 @@ init([N]) ->
     % Mark being able to receive documents with an _access property as a supported feature
     config:enable_feature('access-ready'),
 
-    % Mark if fips is enabled
-    case crypto:info_fips() == enabled of
-        true ->
-            config:enable_feature('fips');
-        false ->
-            ok
-    end,
-
     % read config and register for configuration changes
 
     % just stop if one of the config settings change. couch_server_sup
@@ -344,7 +347,10 @@ terminate(Reason, Srv) ->
     ok.
 
 handle_config_change("couchdb", "database_dir", _, _, N) ->
-    exit(couch_server(N), config_change),
+    case whereis(couch_server(N)) of
+        Pid when is_pid(Pid) -> exit(Pid, {shutdown, config_change});
+        undefined -> ok
+    end,
     remove_handler;
 handle_config_change("couchdb", "update_lru_on_read", "true", _, N) ->
     gen_server:call(couch_server(N), {set_update_lru_on_read, true}),
@@ -365,7 +371,7 @@ handle_config_change("couchdb_engines", _, _, _, N) ->
     {ok, N};
 handle_config_change("admins", _, _, Persist, 1 = N) ->
     % async hashing on couch_server with number 1 only
-    couch_password_hasher:hash(Persist),
+    couch_password_hasher:hash_admin_passwords(Persist),
     {ok, N};
 handle_config_change("httpd", "authentication_handlers", _, _, 1 = N) ->
     couch_httpd:stop(),
@@ -514,13 +520,6 @@ open_async_int(Server, DbName, Options) ->
             Error1
     end.
 
-handle_call(close_lru, _From, #server{lru = Lru} = Server) ->
-    case couch_lru:close(Lru) of
-        {true, NewLru} ->
-            {reply, ok, db_closed(Server#server{lru = NewLru}, [])};
-        false ->
-            {reply, {error, all_dbs_active}, Server}
-    end;
 handle_call(open_dbs_count, _From, Server) ->
     {reply, Server#server.dbs_open, Server};
 handle_call({set_update_lru_on_read, UpdateOnRead}, _From, Server) ->
@@ -728,11 +727,8 @@ handle_cast({close_db_if_idle, DbName}, Server) ->
 handle_cast(Msg, Server) ->
     {stop, {unknown_cast_message, Msg}, Server}.
 
-code_change(_OldVsn, #server{} = State, _Extra) ->
-    {ok, State}.
-
-handle_info({'EXIT', _Pid, config_change}, Server) ->
-    {stop, config_change, Server};
+handle_info({'EXIT', _Pid, {shutdown, config_change}}, Server) ->
+    {stop, {shutdown, config_change}, Server};
 handle_info({'EXIT', Pid, Reason}, Server) ->
     case ets:lookup(couch_dbs_pid_to_name(Server), Pid) of
         [{Pid, DbName}] ->
