@@ -12,7 +12,6 @@
 
 -module(couch_work_queue).
 -behaviour(gen_server).
--vsn(1).
 
 -include_lib("couch/include/couch_db.hrl").
 
@@ -21,7 +20,7 @@
 
 % gen_server callbacks
 -export([init/1, terminate/2]).
--export([handle_call/3, handle_cast/2, code_change/3, handle_info/2]).
+-export([handle_call/3, handle_cast/2, handle_info/2]).
 
 -record(q, {
     queue = queue:new(),
@@ -167,8 +166,5 @@ handle_cast(close, #q{items = 0} = Q) ->
 handle_cast(close, Q) ->
     {noreply, Q#q{close_on_dequeue = true}}.
 
-code_change(_OldVsn, State, _Extra) ->
-    {ok, State}.
-
 handle_info(X, Q) ->
     {stop, X, Q}.
