@@ -27,9 +27,7 @@ start_link() ->
 init([]) ->
     {ok, {
         {one_for_one, 5, 10},
-        [
-            ?CHILD(couch_prometheus_server, worker)
-        ] ++ maybe_start_prometheus_http()
+        [] ++ maybe_start_prometheus_http()
     }}.
 
 maybe_start_prometheus_http() ->
