@@ -39,7 +39,7 @@
 ]).
 -export([get_all_security/2, open_shard/2]).
 -export([compact/1, compact/2]).
--export([get_purge_seq/2, purge_docs/3, set_purge_infos_limit/3]).
+-export([get_purge_seq/2, get_purged_infos/1, purge_docs/3, set_purge_infos_limit/3]).
 
 -export([
     get_db_info/2,
@@ -89,15 +89,24 @@ changes(DbName, Options, StartVector, DbOptions) ->
     DbOpenOptions = Args#changes_args.db_open_options ++ DbOptions,
     case get_or_create_db(DbName, DbOpenOptions) of
         {ok, Db} ->
-            StartSeq = calculate_start_seq(Db, node(), StartVector),
+            StartSeq = calculate_start_seq(Db, config:node_name(), StartVector),
             Enum = fun changes_enumerator/2,
             Opts = [{dir, Dir}],
+            Pending0 =
+                case Dir of
+                    fwd ->
+                        couch_db:count_changes_since(Db, StartSeq);
+                    rev ->
+                        ChangesTotal = couch_db:count_changes_since(Db, 0),
+                        ChangesToEnd = couch_db:count_changes_since(Db, StartSeq),
+                        ChangesTotal - ChangesToEnd
+                end,
             Acc0 = #fabric_changes_acc{
                 db = Db,
                 seq = StartSeq,
                 args = Args,
                 options = Options,
-                pending = couch_db:count_changes_since(Db, StartSeq),
+                pending = Pending0,
                 epochs = couch_db:get_epochs(Db)
             },
             try
@@ -116,36 +125,21 @@ changes(DbName, Options, StartVector, DbOptions) ->
             rexi:stream_last(Error)
     end.
 
-do_changes(Db, StartSeq, Enum, Acc0, Opts) ->
-    #fabric_changes_acc{
-        args = Args
-    } = Acc0,
-    #changes_args{
-        filter = Filter
-    } = Args,
+do_changes(Db, Seq, Enum, #fabric_changes_acc{args = Args} = Acc0, Opts) ->
+    #changes_args{filter_fun = Filter, dir = Dir} = Args,
+
     case Filter of
-        "_doc_ids" ->
-            % optimised code path, we’re looking up all doc_ids in the by-id instead of filtering
-            % the entire by-seq tree to find the doc_ids one by one
-            #changes_args{
-                filter_fun = {doc_ids, Style, DocIds},
-                dir = Dir
-            } = Args,
-            couch_changes:send_changes_doc_ids(
-                Db, StartSeq, Dir, Enum, Acc0, {doc_ids, Style, DocIds}
-            );
-        "_design_docs" ->
-            % optimised code path, we’re looking up all design_docs in the by-id instead of
-            % filtering the entire by-seq tree to find the design_docs one by one
-            #changes_args{
-                filter_fun = {design_docs, Style},
-                dir = Dir
-            } = Args,
-            couch_changes:send_changes_design_docs(
-                Db, StartSeq, Dir, Enum, Acc0, {design_docs, Style}
-            );
+        {doc_ids, _Style, DocIds} ->
+            case length(DocIds) =< couch_changes:doc_ids_limit() of
+                true ->
+                    couch_changes:send_changes_doc_ids(Db, Seq, Dir, Enum, Acc0, Filter);
+                false ->
+                    couch_db:fold_changes(Db, Seq, Enum, Acc0, Opts)
+            end;
+        {design_docs, _Style} ->
+            couch_changes:send_changes_design_docs(Db, Seq, Dir, Enum, Acc0, Filter);
         _ ->
-            couch_db:fold_changes(Db, StartSeq, Enum, Acc0, Opts)
+            couch_db:fold_changes(Db, Seq, Enum, Acc0, Opts)
     end.
 
 all_docs(DbName, Options, Args0) ->
@@ -193,7 +187,8 @@ reduce_view(DbName, DDoc, ViewName, Args0, DbOptions) ->
     Args = fabric_util:upgrade_mrargs(Args0),
     {ok, Db} = get_or_create_db(DbName, DbOptions),
     VAcc0 = #vacc{db = Db},
-    couch_mrview:query_view(Db, DDoc, ViewName, Args, fun reduce_cb/2, VAcc0).
+    Callback = fun(Msg, Acc) -> reduce_cb(Msg, Acc, Args#mrargs.extra) end,
+    couch_mrview:query_view(Db, DDoc, ViewName, Args, Callback, VAcc0).
 
 create_db(DbName) ->
     create_db(DbName, []).
@@ -305,6 +300,10 @@ update_docs(DbName, Docs0, Options) ->
     Docs2 = make_att_readers(Docs1),
     with_db(DbName, Options, {couch_db, update_docs, [Docs2, Options, Type]}).
 
+get_purged_infos(DbName) ->
+    FoldFun = fun({_Seq, _UUID, Id, Revs}, Acc) -> {ok, [{Id, Revs} | Acc]} end,
+    with_db(DbName, [], {couch_db, fold_purge_infos, [0, FoldFun, []]}).
+
 get_purge_seq(DbName, Options) ->
     with_db(DbName, Options, {couch_db, get_purge_seq, []}).
 
@@ -407,7 +406,7 @@ read_repair_filter(DbName, Docs, NodeRevs, Options) ->
 % as the read_repair option to update_docs.
 read_repair_filter(Db, Docs, NodeRevs) ->
     [#doc{id = DocId} | _] = Docs,
-    NonLocalNodeRevs = [NR || {N, _} = NR <- NodeRevs, N /= node()],
+    NonLocalNodeRevs = [NR || {N, _} = NR <- NodeRevs, N /= config:node_name()],
     Nodes = lists:usort([Node || {Node, _} <- NonLocalNodeRevs]),
     NodeSeqs = get_node_seqs(Db, Nodes),
 
@@ -493,14 +492,9 @@ view_cb({meta, Meta}, Acc) ->
     % Map function starting
     ok = rexi:stream2({meta, Meta}),
     {ok, Acc};
-view_cb({row, Row}, Acc) ->
+view_cb({row, Props}, #mrargs{extra = Options} = Acc) ->
     % Adding another row
-    ViewRow = #view_row{
-        id = couch_util:get_value(id, Row),
-        key = couch_util:get_value(key, Row),
-        value = couch_util:get_value(value, Row),
-        doc = couch_util:get_value(doc, Row)
-    },
+    ViewRow = fabric_view_row:from_props(Props, Options),
     ok = rexi:stream2(ViewRow),
     {ok, Acc};
 view_cb(complete, Acc) ->
@@ -508,25 +502,27 @@ view_cb(complete, Acc) ->
     ok = rexi:stream_last(complete),
     {ok, Acc};
 view_cb(ok, ddoc_updated) ->
-    rexi:reply({ok, ddoc_updated}).
+    rexi:reply({ok, ddoc_updated});
+view_cb(ok, insufficient_storage) ->
+    rexi:reply({ok, insufficient_storage}).
 
-reduce_cb({meta, Meta}, Acc) ->
+reduce_cb({meta, Meta}, Acc, _Options) ->
     % Map function starting
     ok = rexi:stream2({meta, Meta}),
     {ok, Acc};
-reduce_cb({row, Row}, Acc) ->
+reduce_cb({row, Props}, Acc, Options) ->
     % Adding another row
-    ok = rexi:stream2(#view_row{
-        key = couch_util:get_value(key, Row),
-        value = couch_util:get_value(value, Row)
-    }),
+    ViewRow = fabric_view_row:from_props(Props, Options),
+    ok = rexi:stream2(ViewRow),
     {ok, Acc};
-reduce_cb(complete, Acc) ->
+reduce_cb(complete, Acc, _Options) ->
     % Finish view output
     ok = rexi:stream_last(complete),
     {ok, Acc};
-reduce_cb(ok, ddoc_updated) ->
-    rexi:reply({ok, ddoc_updated}).
+reduce_cb(ok, ddoc_updated, _Options) ->
+    rexi:reply({ok, ddoc_updated});
+reduce_cb(ok, insufficient_storage, _Options) ->
+    rexi:reply({ok, insufficient_storage}).
 
 changes_enumerator(#full_doc_info{} = FDI, Acc) ->
     changes_enumerator(couch_doc:to_doc_info(FDI), Acc);
@@ -544,44 +540,57 @@ changes_enumerator(DocInfo, Acc) ->
         pending = Pending,
         epochs = Epochs
     } = Acc,
-    #doc_info{id = Id, high_seq = Seq, revs = [#rev_info{deleted = Del} | _]} = DocInfo,
-    case [X || X <- couch_changes:filter(Db, DocInfo, Filter), X /= null] of
-        [] ->
-            ChangesRow =
+    Opts =
+        case Conflicts of
+            true -> [conflicts | DocOptions];
+            false -> DocOptions
+        end,
+    Seq = DocInfo#doc_info.high_seq,
+    {Docs0, Revs} = couch_changes:filter(Db, DocInfo, Filter, IncludeDocs, Conflicts),
+    Changes = [X || X <- Revs, X /= null],
+    Docs = [X || X <- Docs0, X /= null],
+    ChangesRow =
+        case {Changes, Docs, IncludeDocs} of
+            {[], _, _} ->
                 {no_pass, [
                     {pending, Pending - 1},
                     {seq, {Seq, uuid(Db), couch_db:owner_of(Epochs, Seq)}}
                 ]};
-        Results ->
-            Opts =
-                if
-                    Conflicts -> [conflicts | DocOptions];
-                    true -> DocOptions
-                end,
-            ChangesRow =
-                {change, [
-                    {pending, Pending - 1},
-                    {seq, {Seq, uuid(Db), couch_db:owner_of(Epochs, Seq)}},
-                    {id, Id},
-                    {changes, Results},
-                    {deleted, Del}
-                    | if
-                        IncludeDocs -> [doc_member(Db, DocInfo, Opts, Filter)];
-                        true -> []
-                    end
-                ]}
-    end,
+            {_, _, false} ->
+                changes_row(Changes, [], DocInfo, Acc);
+            {_, [], true} ->
+                Docs1 = [doc_member(Db, DocInfo, Opts, Filter)],
+                changes_row(Changes, Docs1, DocInfo, Acc);
+            {_, [Doc | _], true} ->
+                Docs1 = [get_json_doc(Doc, Opts, Filter)],
+                changes_row(Changes, Docs1, DocInfo, Acc)
+        end,
     ok = rexi:stream2(ChangesRow),
     {ok, Acc#fabric_changes_acc{seq = Seq, pending = Pending - 1}}.
 
+changes_row(Changes, Docs, DocInfo, Acc) ->
+    #fabric_changes_acc{db = Db, pending = Pending, epochs = Epochs} = Acc,
+    #doc_info{id = Id, high_seq = Seq, revs = [#rev_info{deleted = Del} | _]} = DocInfo,
+    {change, [
+        {pending, Pending - 1},
+        {seq, {Seq, uuid(Db), couch_db:owner_of(Epochs, Seq)}},
+        {id, Id},
+        {changes, Changes},
+        {deleted, Del}
+        | Docs
+    ]}.
+
 doc_member(Shard, DocInfo, Opts, Filter) ->
     case couch_db:open_doc(Shard, DocInfo, [deleted | Opts]) of
         {ok, Doc} ->
-            {doc, maybe_filtered_json_doc(Doc, Opts, Filter)};
+            get_json_doc(Doc, Opts, Filter);
         Error ->
             Error
     end.
 
+get_json_doc(Doc, Opts, Filter) ->
+    {doc, maybe_filtered_json_doc(Doc, Opts, Filter)}.
+
 maybe_filtered_json_doc(Doc, Opts, {selector, _Style, {_Selector, Fields}}) when
     Fields =/= nil
 ->
@@ -697,7 +706,7 @@ uuid(Db) ->
     binary:part(Uuid, {0, Prefix}).
 
 -ifdef(TEST).
--include_lib("eunit/include/eunit.hrl").
+-include_lib("couch/include/couch_eunit.hrl").
 
 maybe_filtered_json_doc_no_filter_test() ->
     Body = {[{<<"a">>, 1}]},
@@ -714,4 +723,94 @@ maybe_filtered_json_doc_with_filter_test() ->
     {JDocProps} = maybe_filtered_json_doc(Doc, [], Filter),
     ?assertEqual(JDocProps, [{<<"a">>, 1}]).
 
+view_cb_test_() ->
+    {
+        foreach,
+        fun() -> meck:new(rexi) end,
+        fun(_) -> meck:unload() end,
+        [
+            ?TDEF_FE(t_view_cb_meta),
+            ?TDEF_FE(t_view_cb_row_record),
+            ?TDEF_FE(t_view_cb_row_map),
+            ?TDEF_FE(t_view_cb_complete),
+            ?TDEF_FE(t_view_cb_ddoc_updated),
+            ?TDEF_FE(t_view_cb_insufficient_storage)
+        ]
+    }.
+
+t_view_cb_meta(_) ->
+    meck:expect(rexi, stream2, [{meta, meta}], meck:val(ok)),
+    ?assertEqual({ok, acc}, view_cb({meta, meta}, acc)).
+
+t_view_cb_row_record(_) ->
+    Acc = #mrargs{},
+    Props = [{id, id}, {key, key}, {value, value}],
+    ViewRow = #view_row{id = id, key = key, value = value},
+    meck:expect(rexi, stream2, [ViewRow], meck:val(ok)),
+    ?assertEqual({ok, Acc}, view_cb({row, Props}, Acc)).
+
+t_view_cb_row_map(_) ->
+    Acc = #mrargs{extra = [{view_row_map, true}]},
+    Props = [{id, id}, {key, key}, {value, value}],
+    ViewRow = {view_row, #{id => id, key => key, value => value}},
+    meck:expect(rexi, stream2, [ViewRow], meck:val(ok)),
+    ?assertEqual({ok, Acc}, view_cb({row, Props}, Acc)).
+
+t_view_cb_complete(_) ->
+    meck:expect(rexi, stream_last, [complete], meck:val(ok)),
+    ?assertEqual({ok, acc}, view_cb(complete, acc)).
+
+t_view_cb_ddoc_updated(_) ->
+    meck:expect(rexi, reply, [{ok, ddoc_updated}], meck:val(ok)),
+    ?assertEqual(ok, view_cb(ok, ddoc_updated)).
+
+t_view_cb_insufficient_storage(_) ->
+    meck:expect(rexi, reply, [{ok, insufficient_storage}], meck:val(ok)),
+    ?assertEqual(ok, view_cb(ok, insufficient_storage)).
+
+reduce_cb_test_() ->
+    {
+        foreach,
+        fun() -> meck:new(rexi) end,
+        fun(_) -> meck:unload() end,
+        [
+            ?TDEF_FE(t_reduce_cb_meta),
+            ?TDEF_FE(t_reduce_cb_row_record),
+            ?TDEF_FE(t_reduce_cb_row_map),
+            ?TDEF_FE(t_reduce_cb_complete),
+            ?TDEF_FE(t_reduce_cb_ddoc_updated),
+            ?TDEF_FE(t_reduce_cb_insufficient_storage)
+        ]
+    }.
+
+t_reduce_cb_meta(_) ->
+    meck:expect(rexi, stream2, [{meta, meta}], meck:val(ok)),
+    ?assertEqual({ok, acc}, reduce_cb({meta, meta}, acc, options)).
+
+t_reduce_cb_row_record(_) ->
+    Options = [],
+    Props = [{id, id}, {key, key}, {value, value}],
+    ViewRow = #view_row{id = id, key = key, value = value},
+    meck:expect(rexi, stream2, [ViewRow], meck:val(ok)),
+    ?assertEqual({ok, acc}, reduce_cb({row, Props}, acc, Options)).
+
+t_reduce_cb_row_map(_) ->
+    Options = [{view_row_map, true}],
+    Props = [{id, id}, {key, key}, {value, value}],
+    ViewRow = {view_row, #{id => id, key => key, value => value}},
+    meck:expect(rexi, stream2, [ViewRow], meck:val(ok)),
+    ?assertEqual({ok, acc}, reduce_cb({row, Props}, acc, Options)).
+
+t_reduce_cb_complete(_) ->
+    meck:expect(rexi, stream_last, [complete], meck:val(ok)),
+    ?assertEqual({ok, acc}, reduce_cb(complete, acc, options)).
+
+t_reduce_cb_ddoc_updated(_) ->
+    meck:expect(rexi, reply, [{ok, ddoc_updated}], meck:val(ok)),
+    ?assertEqual(ok, reduce_cb(ok, ddoc_updated, options)).
+
+t_reduce_cb_insufficient_storage(_) ->
+    meck:expect(rexi, reply, [{ok, insufficient_storage}], meck:val(ok)),
+    ?assertEqual(ok, reduce_cb(ok, insufficient_storage, options)).
+
 -endif.
