@@ -12,36 +12,26 @@
 
 -module(fabric_db_create_tests).
 
-
 -include_lib("couch/include/couch_eunit.hrl").
--include_lib("couch/include/couch_db.hrl").
 -include_lib("mem3/include/mem3.hrl").
 
-
--define(TDEF(A), {atom_to_list(A), fun A/0}).
-
-
 main_test_() ->
     {
         setup,
         fun setup/0,
         fun teardown/1,
-        [
+        with([
             ?TDEF(t_handle_shard_doc_conflict)
-        ]
+        ])
     }.
 
-
 setup() ->
     test_util:start_couch([fabric]).
 
-
 teardown(Ctx) ->
-    meck:unload(),
     test_util:stop_couch(Ctx).
 
-
-t_handle_shard_doc_conflict() ->
+t_handle_shard_doc_conflict(_) ->
     DbName = ?tempdb(),
     meck:new(mem3, [passthrough]),
     meck:new(fabric_util, [passthrough]),
@@ -50,4 +40,5 @@ t_handle_shard_doc_conflict() ->
         [#shard{dbname = DbName}]
     ]),
     meck:expect(fabric_util, recv, 4, {error, conflict}),
-    ?assertEqual({error, file_exists}, fabric_db_create:go(DbName, [])).
+    ?assertEqual({error, file_exists}, fabric_db_create:go(DbName, [])),
+    meck:unload().
