@@ -24,7 +24,7 @@ describe('CORS actions', () => {
     const localNode = 'node2@127.0.0.1';
     const baseURL = 'http://localhost:8000/#_config/couchdb@localhost/cors';
     const dispatch = sinon.stub();
-    const spyUpdateEnableCorsToHttpd = sinon.stub(CorsAPI, 'updateEnableCorsToHttpd');
+    const spyUpdateEnableCorsToHttpd = sinon.stub(CorsAPI, 'updateEnableCorsToChttpd');
     const spyUpdateCorsOrigins = sinon.stub(CorsAPI, 'updateCorsOrigins');
     const spyUpdateCorsCredentials = sinon.stub(CorsAPI, 'updateCorsCredentials');
     const spyUpdateCorsHeaders = sinon.stub(CorsAPI, 'updateCorsHeaders');
