@@ -29,8 +29,9 @@ module.exports = {
       .loginToGUI()
       .url(baseUrl + '/#replication')
       .waitForElementNotPresent('.load-lines', waitTime, true)
-      .waitForElementPresent('.replication__filter', waitTime, true)
-      .clickWhenVisible('a[href="#/database/_replicator/existing-doc-id-view-doc"]')
+      .waitForElementPresent('#replication-filter-group', waitTime, true)
+      .assert.elementsCount('.replication__table-row', 1)
+      .clickWhenVisible('button[title="Edit replication document"]')
       .waitForElementNotPresent('.load-lines', waitTime, true)
       .waitForElementPresent('#editor-container', waitTime, true)
       .end();
@@ -52,12 +53,12 @@ module.exports = {
       .loginToGUI()
       .url(baseUrl + '/#replication')
       .waitForElementNotPresent('.load-lines', waitTime, true)
-      .waitForElementPresent('.replication__filter', waitTime, true)
-      .clickWhenVisible('a[title="Edit replication"]')
+      .waitForElementPresent('#replication-filter-group', waitTime, true)
+      .clickWhenVisible('button[title="Edit replication"]')
       .waitForElementNotPresent('.load-lines', waitTime, true)
-      .waitForElementPresent('.replication__section', waitTime, true)
+      .waitForElementPresent('#replication-options-replication-doc', waitTime, true)
       .pause(10000)
-      .assert.valueContains(".replication__doc-name-input", replicatorDoc._id)
+      .assert.valueContains("#replication-options-replication-doc", replicatorDoc._id)
       .end();
   },
 
@@ -84,14 +85,15 @@ module.exports = {
       .loginToGUI()
       .url(baseUrl + '/#replication')
       .waitForElementNotPresent('.load-lines', waitTime, true)
-      .waitForElementVisible('.replication__filter-input', waitTime, true)
-      .setValue('.replication__filter-input', 'filter1')
-      .waitForElementNotPresent('a[href="#/database/_replicator/existing-doc-filter2"]', waitTime, true)
-      .clearValue('.replication__filter-input')
-      .setValue('.replication__filter-input', 'filter')
-      .waitForElementPresent('a[href="#/database/_replicator/existing-doc-filter2"]', waitTime, true)
+      .waitForElementVisible('#replication-filter-input', waitTime, true)
+      .setValue('#replication-filter-input', 'filter1')
+      .waitForElementNotPresent('button[aria-label="Delete document existing-doc-filter2"]', waitTime, true)
+      .clearValue('#replication-filter-input')
+      .setValue('#replication-filter-input', 'filter')
+      .waitForElementPresent('button[aria-label="Delete document existing-doc-filter2"]', waitTime, true)
       .end();
   },
+
   "Action click doesn't change doc's order": client =>{
     const waitTime = client.globals.maxWaitTime;
     const baseUrl = client.options.launch_url;
@@ -121,7 +123,7 @@ module.exports = {
       .getText(firstRowSelector + ' td:nth-of-type(2)', function(result) {
         firstDoc = result.value;
       })
-      .clickWhenVisible(firstRowSelector + ' .replication__row-btn.icon-trash', waitTime, true)
+      .clickWhenVisible(firstRowSelector + ' button[title="Delete document"]', waitTime, true)
       .clickWhenVisible('.replication_delete-doc-modal.modal-dialog .modal-footer .cancel-link', waitTime, true)
       .waitForElementVisible(firstRowSelector, waitTime, true)
       .getText(firstRowSelector + ' td:nth-of-type(2)', function(result) {
@@ -129,5 +131,79 @@ module.exports = {
           'Checking if the order was reserved if no documents were sorted');
       })
       .end();
+  },
+
+  "Change number of replications displayed": client =>{
+    const waitTime = client.globals.maxWaitTime;
+    const baseUrl = client.options.launch_url;
+
+    const replicatorDoc1 = {
+      _id: 'existing-doc-id-display',
+      source: "http://source-db.com",
+      target: "http://target-db.com"
+    };
+
+    const replicatorDoc2 = {
+      _id: 'existing-doc-id-display2',
+      source: "http://source-db2.com",
+      target: "http://target-db.com"
+    };
+
+    const replicatorDoc3 = {
+      _id: 'existing-doc-id-display3',
+      source: "http://source-db3.com",
+      target: "http://target-db.com"
+    };
+
+    const replicatorDoc4 = {
+      _id: 'existing-doc-id-display4',
+      source: "http://source-db4.com",
+      target: "http://target-db.com"
+    };
+
+    const replicatorDoc5 = {
+      _id: 'existing-doc-id-display5',
+      source: "http://source-db5.com",
+      target: "http://target-db.com"
+    };
+
+    const replicatorDoc6 = {
+      _id: 'existing-doc-id-display6',
+      source: "http://source-db6.com",
+      target: "http://target-db.com"
+    };
+
+    client
+      .deleteDatabase('_replicator')
+      .createDatabase('_replicator')
+      .createDocument(replicatorDoc1._id, '_replicator', replicatorDoc1)
+      .createDocument(replicatorDoc2._id, '_replicator', replicatorDoc2)
+      .createDocument(replicatorDoc3._id, '_replicator', replicatorDoc3)
+      .createDocument(replicatorDoc4._id, '_replicator', replicatorDoc4)
+      .createDocument(replicatorDoc5._id, '_replicator', replicatorDoc5)
+      .createDocument(replicatorDoc6._id, '_replicator', replicatorDoc6)
+      .loginToGUI()
+      .url(baseUrl + '/#replication')
+      .waitForElementNotPresent('.load-lines', waitTime, true)
+      .waitForElementPresent('.replication__table-row', waitTime, true)
+      .getText('.current-replications', function(result) {
+        this.verify.ok(result.value === "Showing replications 1 - 6");
+      })
+      .assert.elementsCount('.replication__table-row', 6)
+      .clickWhenVisible('select[id="select-per-page"] option[value="5"]')
+      .waitForElementNotPresent('.load-lines', waitTime, true)
+      .waitForElementPresent('.replication__table-row', waitTime, true)
+      .getText('.current-replications', function(result) {
+        this.verify.ok(result.value === "Showing replications 1 - 5");
+      })
+      .assert.elementsCount('.replication__table-row', 5)
+      .clickWhenVisible('select[id="select-per-page"] option[value="25"]')
+      .waitForElementNotPresent('.load-lines', waitTime, true)
+      .waitForElementPresent('.replication__table-row', waitTime, true)
+      .getText('.current-replications', function(result) {
+        this.verify.ok(result.value === "Showing replications 1 - 6");
+      })
+      .assert.elementsCount('.replication__table-row', 6)
+      .end();
   }
 };
